//
//  ViewController.swift
//  TextLCD
//
//  Created by Phidgets on 2018-02-02.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController, UITextFieldDelegate {

    let ch = LCD()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var initButton: UIButton!
    @IBOutlet var customSwitch: UISwitch!
    @IBOutlet var cursorBlinkSwitch: UISwitch!
    @IBOutlet var cursorSwitch: UISwitch!
    @IBOutlet var backlightLabel: UILabel!
    @IBOutlet var backlightSlider: UISlider!
    @IBOutlet var contrastSlider: UISlider!
    @IBOutlet var contrastLabel: UILabel!
    @IBOutlet var displayText: [UITextField]!
    @IBOutlet var settingsStack: UIStackView!
    @IBOutlet var stackView: UIStackView!
    @IBOutlet var screenSizeButton: UIButton!
    
    let heart:[UInt8] = [0,0,0,0,0,
                         0,1,0,1,0,
                         1,1,1,1,1,
                         1,1,1,1,1,
                         0,1,1,1,0,
                         0,0,1,0,0,
                         0,0,0,0,0,
                         0,0,0,0,0]
    
    let smiley:[UInt8] =  [0,0,0,0,0,
                           0,1,0,1,0,
                           0,0,0,0,0,
                           0,0,1,0,0,
                           0,0,0,0,0,
                           1,0,0,0,1,
                           0,1,1,1,0,
                           0,0,0,0,0]

    
    var pickerArray = [ "dimensions_1x8",
                        "dimensions_2x8",
                        "dimensions_1x16",
                        "dimensions_2x16",
                        "dimensions_4x16",
                        "dimensions_2x20",
                        "dimensions_4x20",
                        "dimensions_2x24",
                        "dimensions_1x40",
                        "dimensions_2x40",
                        "dimensions_4x40"
    ]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! LCD
            
            let deviceID = try ch.getDeviceID()
            
            let minContrast = try Float(attachedDevice.getMinContrast())
            let maxContrast = try Float(attachedDevice.getMaxContrast())
            let contrast = try Float(attachedDevice.getContrast())
            
            let minBacklight = try Float(attachedDevice.getMinBacklight())
            let maxBacklight = try Float(attachedDevice.getMaxBacklight())
            let backlight = try Float(attachedDevice.getBacklight())

            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.settingsStack.isHidden = false
                
                self.contrastSlider.minimumValue = minContrast
                self.contrastSlider.maximumValue = maxContrast
                self.contrastSlider.value = contrast
                self.contrastLabel.text = String(format:"%.2f", contrast)
                
                self.backlightSlider.minimumValue = minBacklight
                self.backlightSlider.maximumValue = maxBacklight
                self.backlightSlider.value = backlight
                self.backlightLabel.text = String(format:"%.2f",backlight)
                
                if(deviceID == DeviceID.PN_1204){
                    self.screenSizeButton.isHidden = false
                    self.initButton.isHidden = false
                }
                else{
                    self.screenSizeButton.isHidden = true
                    self.initButton.isHidden = true
                }
                self.updateDisplay()
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
                self.settingsStack.isHidden = false
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    //MARK: GUI controls
    @IBAction func clear(_ sender: Any) {
        for textfield in displayText{
            textfield.text = ""
        }
        do{
            try ch.clear()
            try ch.flush()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func initialize(_ sender: Any) {
        do{
            try ch.initialize()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setContrast(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setContrast(Double(slider.value))
            DispatchQueue.main.async {
                self.contrastLabel.text = String(format:"%.2f",slider.value)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setBacklight(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setBacklight(Double(slider.value))
            DispatchQueue.main.async {
                self.backlightLabel.text = String(format:"%.2f",slider.value)
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setCursor(_ sender: Any) {
        let swtch = sender as! UISwitch
        do{
            try ch.setCursorOn(swtch.isOn)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setCursorBlink(_ sender: Any) {
        let swtch = sender as! UISwitch
        do{
            try ch.setCursorBlink(swtch.isOn)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setCustom(_ sender: Any) {
        do{
			try ch.setCharacterBitmap(font: LCDFont.dimensions_5x8, character: "\u{8}", bitmap: heart)
            try ch.setCharacterBitmap(font: LCDFont.dimensions_5x8, character: "\u{9}", bitmap: smiley)
            try ch.writeText(font: LCDFont.dimensions_5x8, xPosition: 0, yPosition: 0, text: "\u{8} Custom chars! \u{9}")
			try ch.flush()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func editingChanged(_ sender: Any) {
        let textfield = sender as! UITextField
        do{
            try ch.clear()
            for field in displayText{
                if(field.isEnabled){
                    try ch.writeText(font: LCDFont.dimensions_5x8, xPosition: 0, yPosition: field.tag, text: field.text!)
                }
            }
            //write here again so cursor position is correct
            try ch.writeText(font: LCDFont.dimensions_5x8, xPosition: 0, yPosition: textfield.tag, text: textfield.text!)
            try ch.flush()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: navigation
    @IBAction func pickerExit(_ segue: UIStoryboardSegue){
        dimBackground()
        updateDisplay()
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
        else if segue.identifier == "pickerSegue"{
            let vc:PickerViewController = segue.destination as! PickerViewController
            dimBackground()
            vc.titleName = "Screen Size"
            vc.pickerArray = pickerArray
            vc.ch = ch
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func dimBackground(){
        self.view.alpha = (self.view.alpha == 1.0) ? 0.5 : 1.0
    }
    
    func updateDisplay(){
        do{
            for textField in displayText{
                textField.isEnabled = false
                textField.backgroundColor = #colorLiteral(red: 0.9125197593, green: 0.9125197593, blue: 0.9125197593, alpha: 1)
            }
            let height = try ch.getHeight()
            for i in 0 ..< height{
                displayText[i].isEnabled = true
                displayText[i].backgroundColor = UIColor.clear
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Text field
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        self.view.endEditing(true)
        return false
    }
}

